// Author : Pierre Vuillemin (2017)
// License : GNU GPL

function [A, B] = sopi_extractAffineForm(fun, problem)
   // Extracts an affine form A*x + B from a variable.
   //
   // Calling Sequence
   //    [A, B] = sopi_extractAffineForm(var, problem)
   //
   // Parameters
   //    var      : the variable to parse (sopiVar)
   //    problem  : the associated optimisation problem (sopiPro, can be empty)
   //    A        : the linear mapping (matrix)
   //    B        : the constant part (matrix)
   // Description
   //    Considering a function fun(x1, .., xn) where the xi are variables of an
   //    optimisation problem, this routines extracts the matrix A such that the
   //    affine part of the function is written as A * x + B, x containing the
   //    vectorized xi in a coherent order w.r.t. the optimisation problem. It
   //
   
   // Linear part
   if argn(2)<2 then
      // Emulating a sopi_problem to order variables
      varList           = fun.depends
      [idxList, nvar]   = sopi_orderVariables(varList)
      problem           = struct('varList',varList,'varIdx',idxList, 'nvar', nvar)
   end
   // First, the linear parts are extracted, out contains the matrices so that
   // Li * xi * Ri, or Li * xi' * Ri
   nvar  = problem.nvar
   out   = sopi_extractPolyPart(fun, 1)
   A     = []
   for o = out
      var = o(2)(1)
      if var.isElementary
         idx   = sopi_getVarIdxInProblem(problem, var)
         P     = 1;
      elseif var.operator == 'transpose'
         idx   = sopi_getVarIdxInProblem(problem, var.args(1))
         [m,n] = size(var)
         P     = sopi_vecTranspose(m,n)
      end
      // Here one use the following relation: vec(L * x * R) = kron(R', L) * vec(x)
      // + when transposed variables are returned from extractPolyPart, the
      // matrix P is such that vec(x') = P vec(x)
      L        = o(1)(1)
      R        = o(1)(2)
      Ai       = (R' .*. L) * P
      if isempty(A) then
         A = zeros(size(Ai,1), nvar)
      end
      A(:,idx) = A(:,idx) + Ai
   end
   // Constant part
   if argn(1) >1 then
      B = sopi_extractConstantPart(fun)
   end
endfunction
